
/**
 * office数据文件导入脚本
 * 
 * file MultipartFile 外部传进来的参数
 */
import groovy.ui.SystemOutputInterceptor
import com.kdayun.z1.core.context.Context
import com.kdayun.z1.core.message.MessageUtil

import java.util.UUID;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFDateUtil;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.apache.poi.xwpf.extractor.XWPFWordExtractor;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.apache.poi.xwpf.usermodel.XWPFTable;
import org.apache.poi.xwpf.usermodel.XWPFTableCell;
import org.apache.poi.xwpf.usermodel.XWPFTableRow;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;

/**
 * 表头的索引
 * 注意:从零开始
 */
headerIndex=1

/**
 * 模板的列
 * 注意:这个客户提供的word模板的类字段 ,可修改模板这里必须修改,不然通不过校验
 * fields 列
 * rows 数据行
 * field 字段名(数据映射的字段名,无需更新设置成""或者null), regx:正则表达式校验合法 ,nullable: 允许为空  
 * 			colIndex:列的位置(正常情况与cell[1]相等多表导致这个值与cell[1]不等)
 * 			cell: 表头单元格的坐标 [row,col], data:文件内保存的字段名,用于校验表头是否与提供的一致
 */ 
FIELDS_MAP= [
	"fields":[	
		"序号":		    ["field":"" ,"regx":"" , "nullable":true,colIndex:0, cell:[0,0],data:null]
		,"姓名":		    ["field":"XINGM" ,"regx":"" , "nullable":false,colIndex:1, cell:[0,1],data:null]
		,"现机构及职务":    ["field":"XIANDWJZW" ,"regx":"" , "nullable":false,colIndex:2, cell:[0,2],data:null]
		,"性别":		    ["field":"XINGB" ,"regx":"" , "nullable":false,colIndex:3, cell:[0,3],data:null]
		,"出生年月":	    ["field":"CHUSNY" ,"regx":"\\d{4}.\\d{1,2}" , "nullable":false,colIndex:4, cell:[0,4],data:null]
		,"民族":		    ["field":"MINZ" ,"regx":"" , "nullable":false,colIndex:5, cell:[0,5],data:null]
		,"籍贯":		    ["field":"JIG" ,"regx":"" , "nullable":false,colIndex:6, cell:[0,6],data:null]
		,"参加工作时间":    ["field":"CANJGZSJ" ,"regx":"\\d{4}.\\d{1,2}" , "nullable":false,colIndex:7, cell:[0,7],data:null]
		,"加入中共时间":    ["field":"JIARZGSJ" ,"regx":"\\d{4}.\\d{1,2}" , "nullable":false,colIndex:8, cell:[0,8],data:null]
		,"全日学历":	    	["field":"QUANRXL" ,"regx":"" , "nullable":false,colIndex:9, cell:[1,9],data:null]
		,"全日学位":	    	["field":"QUANRXW" ,"regx":"" , "nullable":false,colIndex:10, cell:[1,10],data:null]
		,"全日毕业院校及专业":	["field":"QUANRBYYXJZY" ,"regx":"" , "nullable":false,colIndex:11, cell:[1,11],data:null]
		,"在职学历":	    	["field":"ZAIZXL" ,"regx":"" , "nullable":true,colIndex:12, cell:[1,12],data:null]
		,"在职学位":	    	["field":"ZAIZXW" ,"regx":"" , "nullable":true,colIndex:13, cell:[1,13],data:null]
		,"在职毕业院校及专业":	["field":"ZAIZBYYXJZY" ,"regx":"" , "nullable":true,colIndex:14, cell:[1,14],data:null]
		,"专业技术职务":    ["field":"ZHUANYJSZW" ,"regx":"" , "nullable":true,colIndex:15, cell:[0,11],data:null]
		,"任现职时间":	    ["field":"RENXZSJ" ,"regx":"" , "nullable":false,colIndex:16, cell:[0,12],data:null]
		,"备注":		    ["field":"BEIZ" ,"regx":"" , "nullable":true,colIndex:17, cell:[0,13],data:null]
	]
	,"rows":[]
]

INSERST_SQL="INSERT INTO CESB_RENYDR(RWID,XINGM,XIANDWJZW,XINGB,CHUSNY,MINZ,JIG,CANJGZSJ,JIARZGSJ,QUANRXL,QUANRXW,QUANRBYYXJZY,ZAIZXL,ZAIZXW,ZAIZBYYXJZY,ZHUANYJSZW,RENXZSJ,BEIZ,SYS_CREATOR,SYS_CREATETIME) VALUES (#{RWID},#{XINGM},#{XIANDWJZW},#{XINGB},#{CHUSNY},#{MINZ},#{JIG},#{CANJGZSJ},#{JIARZGSJ},#{QUANRXL},#{QUANRXW},#{QUANRBYYXJZY},#{ZAIZXL},#{ZAIZXW},#{ZAIZBYYXJZY},#{ZHUANYJSZW},#{RENXZSJ},#{BEIZ},#{SYS_CREATOR},#{SYS_CREATETIME})"
SELECT_SQL="SELECT 1 FROM CESB_RENYDR WHERE XINGM=#{XINGM}  AND CHUSNY=#{CHUSNY}"

/**
 * 保存的原始数据
 */
rawData=[]

/**
 * 校验文件的格式合法性
 * 文件的格式 后缀
 * @return true 通过 则反之
 */
def void checkImportFile() throws Exception {
	System.out.println("校验文件合法性...");
	if(null==file) {
		throw new Exception("输入文件不允许为空")
	}
	
}

/**
 * 校验文件的文数据合法
 * @return
 */
def void checkImportData() throws Exception {
	System.out.println("校验数据合法性...");
	if(FIELDS_MAP.rows.size()==0) {
		throw new Exception("无数据可以导入")
	}
	checkFields();
	checkRows();
	
}
/**
 * 校验列
 */
def checkFields() {
	for ( e in FIELDS_MAP.fields ) {
		if(StringUtils.isBlank(e.value.data)) {
			throw new Exception("导入的文件不存在数据列["+e.key+"]")
		}
	}
}
/**
 * 校验行
 */
def checkRows() {
	for(int rowindex=0;rowindex<FIELDS_MAP.rows.size();rowindex++) {
		row=FIELDS_MAP.rows.get(rowindex);
		for ( e in FIELDS_MAP.fields ) {					
			checkRowData(rowindex,row,e)
		}
	}
}

/**
 * 校验行的数据
 */
def void checkRowData(Integer rowindex,ArrayList<String> row,def field) {
	 
	 value=row[field.value.colIndex];
	 cellTitle="行["+rowindex+"] 列["+field.value.colIndex+"]"+field.key +" "; 
	 regx=field.value.regx;
	 nullable=field.value.nullable
	 
	 System.out.println("校验数据 "+cellTitle+" "+value);
	 if(!nullable) {
		 if(StringUtils.isBlank(value)) {
			 throw new Exception( cellTitle+"的数据不允许为空")
		 }
		 strValue= value.toString();
		 if(StringUtils.isNotBlank(regx)) {
			 if(!strValue.matches(regx).find()) {
				 throw new Exception( cellTitle+"的数据格式不对")
			 }
		 }		 
	 }else {
		 if(StringUtils.isNotBlank(regx) && null!=value) {
			 strValue= value.toString();
			 if(!strValue.matches(regx).find()) {
				 throw new Exception( cellTitle+"的数据格式不对")
			 }
		 }
	 
	 }
}
/**
 * 读取数据文件的数据
 * @return
 */
def getData() {
	System.out.println("读取文件数据...");
	readWord7();		
	readFields();
	readRows();		
	System.out.println("读取文件数据结果 row:"+FIELDS_MAP.rows.size()+" field:"+FIELDS_MAP.fields.size());
}
/**
 * 检查是否存在行
 */
def isExistsRow(variants) {
	List ret= cxt.exceSelectSql(SELECT_SQL,variants);
	return ret.size()>0;
}
def insertData(){
	System.out.println("写入数据到数据库...");
	variants=[]
	def user=Context.getInstance().securityService.getCurrentUser();
	if(null==user) {
		throw new Exception("必须选登录才能导入数据")
	}
	for(int j;j<FIELDS_MAP.rows.size();j++) {
		def row=FIELDS_MAP.rows[j];
		def vrow=[:];
		variants.add(vrow);
		//给一个组件guid
		vrow.put("RWID",UUID.randomUUID().toString().replaceAll("-", "").toUpperCase());
		vrow.put("SYS_CREATOR",user.getId());
		vrow.put("SYS_CREATETIME",new Date())
		for ( e in FIELDS_MAP.fields ) {
			if(StringUtils.isNotBlank(e.value.field)) {
			  vrow.put(e.value.field,row[e.value.colIndex]);
			}
		}
	}
	System.out.println("写入数据到数据库:"+JSON.toJSONString(variants));
	for(int j;j<FIELDS_MAP.rows.size();j++) {
		def row=FIELDS_MAP.rows[j];
		try {
			if(isExistsRow(variants[j])) {
				throw new Exception("第["+j+"]行数据已经存在,检查是否重复导入数据")
			}else {
				cxt.exceSelectSql(INSERST_SQL,variants[j]);
			}
		}
		catch(e) {
			throw new Exception("第["+j+"行]数据写入问题:"+e.toString());
		}
	}
	
}

/**
 * 得到table单元格的值
 */
def getCellValue(Integer rowIndex,Integer colIndex){
	return rawData.get(rowIndex).get(colIndex);
}

/**
 * 读取数据列
 */
def readFields() {
	if(rawData.size()==0) {
		throw new Exception( "无数据可以导入")
	}
	System.out.println("读取数据列...");
	for ( e in FIELDS_MAP.fields ) {
		System.out.println("读取数据列"+JSON.toJSONString(e));
		if(e.value.cell!=null) {
			rowIndex= e.value.cell[0]
			colIndex= e.value.cell[1]
			e.value.data=getCellValue(rowIndex,colIndex).replaceAll("\\s*", "");
		}
	}
		
}

/**
 * 读取数据行
 */
def readRows() {
	if(rawData.size()==0) {
		throw new Exception( "无数据可以导入")
	}
	System.out.println("读取数据行...");
	for(int i;i<rawData.size();i++) {
		if(i>headerIndex) {
			def row=[];
			FIELDS_MAP.rows.add(row);		
			for ( e in FIELDS_MAP.fields ) {
				def rowIndex= i
				def colIndex= e.value.colIndex
				if(colIndex!=null) {
					row.add(getCellValue(rowIndex,colIndex))
				}
			}
		}		
	}
	System.out.println("读取数据列"+JSON.toJSONString(FIELDS_MAP));
}

def readWord7(){
	String text = null;
	InputStream inputStream =null;
	XWPFDocument doc =null;
	try {
		inputStream = file.getInputStream();
		doc = new XWPFDocument(inputStream);
		List<XWPFTable> tables = doc.getTables();
		for(int i=0;i<tables.size();i++) {
			List<XWPFTableRow> wordRows = tables.get(i).getRows();
			for(int j=0;j<wordRows.size();j++) {
				XWPFTableRow wrodRow=wordRows.get(j);
				List<XWPFTableCell> tableCells = wrodRow.getTableCells();
				List<String> r=new ArrayList<String>();
				rawData.add(r)
				for(int n=0;n<tableCells.size();n++) {
					XWPFTableCell cell=tableCells.get(n);
					r.add(cell.getText())
				}
			}
		}
		System.out.println("数据:"+JSON.toJSONString(rawData));
	}finally {
		if (doc!=null) {
			try {
				doc.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
}
 
/**
 * 开始导入
 */
def void start() {
	System.out.println("开始导入...");
	checkImportFile()
	getData();
	checkImportData();
	insertData();
}

start();